@@ -3,7 +3,7 @@
 """
 Python client for the Rosette API.
 
-Copyright (c) 2014-2022 Basis Technology Corporation.
+Copyright (c) 2014-2024 Basis Technology Corporation.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -30,7 +30,7 @@ import platform
 
 _APPLICATION_JSON = 'application/json'
 _BINDING_LANGUAGE = 'python'
-_BINDING_VERSION = '1.20.0'
+_BINDING_VERSION = '1.29.0'
 _CONCURRENCY_HEADER = 'x-rosetteapi-concurrency'
 _CUSTOM_HEADER_PREFIX = 'X-RosetteAPI-'
 _CUSTOM_HEADER_PATTERN = re.compile('^' + _CUSTOM_HEADER_PREFIX)
@@ -86,7 +86,7 @@ class RosetteException(Exception):
         return sst + ": " + self.message + ":\n  " + self.response_message
 
 
-class _DocumentParamSetBase(object):
+class _RequestParametersBase(object):
 
     def __init__(self, repertoire):
         self.__params = {}
@@ -113,7 +113,7 @@ class _DocumentParamSetBase(object):
         """serialize keys with values"""
         self.validate()
         values = {}
-        for (key, val) in self.__params.items():
+        for key, val in self.__params.items():
             if val is None:
                 continue
             else:
@@ -135,7 +135,7 @@ def _byteify(value):  # py 3 only
     return byte_array
 
 
-class DocumentParameters(_DocumentParamSetBase):
+class DocumentParameters(_RequestParametersBase):
     """Parameter object for all operations requiring input other than
     translated_name.
     Two fields, C{content} and C{inputUri}, are set via
@@ -152,8 +152,8 @@ class DocumentParameters(_DocumentParamSetBase):
 
     def __init__(self):
         """Create a L{DocumentParameters} object."""
-        _DocumentParamSetBase.__init__(
-            self, ("content", "contentUri", "language", "genre", "profileId"))
+        _RequestParametersBase.__init__(
+            self, ("content", "contentUri", "language", "profileId"))
         self.file_name = ""
         self.use_multipart = False
 
@@ -199,7 +199,7 @@ class DocumentParameters(_DocumentParamSetBase):
         self["content"] = content_as_string
 
 
-class NameTranslationParameters(_DocumentParamSetBase):
+class NameTranslationParameters(_RequestParametersBase):
     """Parameter object for C{name-translation} endpoint.
     The following values may be set by the indexing (i.e.,C{ parms["name"]}) operator.
     The values are all strings (when not C{None}).
@@ -212,7 +212,7 @@ class NameTranslationParameters(_DocumentParamSetBase):
 
     C{targetLangauge} The language into which the name is to be translated.
 
-    C{entityType} The entity type (TBD) of the name.
+    C{entityType} The entity type of the name.  PERSON (default), LOCATION, or ORGANIZATION
 
     C{sourceLanguageOfOrigin} The language of origin of the name.
 
@@ -227,7 +227,7 @@ class NameTranslationParameters(_DocumentParamSetBase):
 
     def __init__(self):
         self.use_multipart = False
-        _DocumentParamSetBase.__init__(
+        _RequestParametersBase.__init__(
             self,
             ("name",
              "targetLanguage",
@@ -236,12 +236,11 @@ class NameTranslationParameters(_DocumentParamSetBase):
              "sourceLanguageOfUse",
              "sourceScript",
              "targetScript",
-             "targetScheme",
-             "genre"))
+             "targetScheme"))
 
     def validate(self):
         """Internal. Do not use."""
-        for option in ("name", "targetLanguage"):  # required
+        for option in "name", "targetLanguage":  # required
             if self[option] is None:
                 raise RosetteException(
                     "missingParameter",
@@ -249,9 +248,12 @@ class NameTranslationParameters(_DocumentParamSetBase):
                     repr(option))
 
 
-class AddressSimilarityParameters(_DocumentParamSetBase):
+class AddressSimilarityParameters(_RequestParametersBase):
     """Parameter object for C{address-similarity} endpoint.
-    All are required.
+
+    C{address1} and C{address2} are required.
+
+    `parameters` is optional.
 
     C{address1} The address to be matched, a C{address} object or address string.
 
@@ -259,15 +261,21 @@ class AddressSimilarityParameters(_DocumentParamSetBase):
 
     The C{address} object contains these optional fields:
       city, island, district, stateDistrict, state, countryRegion, country, worldRegion, postCode, poBox
+
+    `parameters` is a dictionary listing any parameter overrides to include.  For example, `postCodeAddressFieldWeight`.
+    Setting `parameters` is not cumulative.  Define all overrides at once.  If defined multiple times, only the
+    final declaration is used.
+
+    See `examples/address_similarity.py`
     """
 
     def __init__(self):
         self.use_multipart = False
-        _DocumentParamSetBase.__init__(self, ("address1", "address2"))
+        _RequestParametersBase.__init__(self, ("address1", "address2", "parameters"))
 
     def validate(self):
         """Internal. Do not use."""
-        for option in ("address1", "address2"):  # required
+        for option in "address1", "address2":  # required
             if self[option] is None:
                 raise RosetteException(
                     "missingParameter",
@@ -275,9 +283,12 @@ class AddressSimilarityParameters(_DocumentParamSetBase):
                     repr(option))
 
 
-class NameSimilarityParameters(_DocumentParamSetBase):
+class NameSimilarityParameters(_RequestParametersBase):
     """Parameter object for C{name-similarity} endpoint.
-    All are required.
+
+    C{name1} and C{name2} are required.
+
+    `parameters` is optional.
 
     C{name1} The name to be matched, a C{name} object.
 
@@ -285,22 +296,28 @@ class NameSimilarityParameters(_DocumentParamSetBase):
 
     The C{name} object contains these fields:
 
-    C{text} Text of the name, required.
+        C{text} Text of the name, required.
 
-    C{language} Language of the name in ISO639 three-letter code, optional.
+        C{language} Language of the name in ISO639 three-letter code, optional.
 
-    C{script} The ISO15924 code of the name, optional.
+        C{script} The ISO15924 code of the name, optional.
 
-    C{entityType} The entity type, can be "PERSON", "LOCATION" or "ORGANIZATION", optional.
+        C{entityType} The entity type, can be "PERSON", "LOCATION" or "ORGANIZATION", optional.
+
+    `parameters` is a dictionary listing any parameter overrides to include.  For example, `deletionScore`.
+    Setting `parameters` is not cumulative.  Define all overrides at once.  If defined multiple times, only the
+    final declaration is used.
+
+    See `examples/name_similarity.py`
     """
 
     def __init__(self):
         self.use_multipart = False
-        _DocumentParamSetBase.__init__(self, ("name1", "name2"))
+        _RequestParametersBase.__init__(self, ("name1", "name2", "parameters"))
 
     def validate(self):
         """Internal. Do not use."""
-        for option in ("name1", "name2"):  # required
+        for option in "name1", "name2":  # required
             if self[option] is None:
                 raise RosetteException(
                     "missingParameter",
@@ -308,7 +325,7 @@ class NameSimilarityParameters(_DocumentParamSetBase):
                     repr(option))
 
 
-class NameDeduplicationParameters(_DocumentParamSetBase):
+class NameDeduplicationParameters(_RequestParametersBase):
     """Parameter object for C{name-deduplication} endpoint.
     Required:
     C{names} A list of C{name} objects
@@ -317,7 +334,7 @@ class NameDeduplicationParameters(_DocumentParamSetBase):
 
     def __init__(self):
         self.use_multipart = False
-        _DocumentParamSetBase.__init__(self, ("names", "threshold"))
+        _RequestParametersBase.__init__(self, ("names", "threshold"))
 
     def validate(self):
         """Internal. Do not use."""
@@ -328,6 +345,29 @@ class NameDeduplicationParameters(_DocumentParamSetBase):
                 repr("names"))
 
 
+class RecordSimilarityParameters(_RequestParametersBase):
+    """Parameter object for C{record-similarity} endpoint.
+    Required:
+    C{records} The records to be compared; where each left record is compared to the associated right record.
+    C{properties} Parameters used in the call
+    C{fields} The definition of the fields used in the comparison. There must be a minimum of 1 field and
+    can have a maximum of 5 fields.
+    """
+
+    def __init__(self):
+        self.use_multipart = False
+        _RequestParametersBase.__init__(self, ("fields", "properties", "records"))
+
+    def validate(self):
+        """Internal. Do not use."""
+        for option in "fields", "properties", "records":  # required
+            if self[option] is None:
+                raise RosetteException(
+                    "missingParameter",
+                    "Required Record Similarity parameter is missing: " + option,
+                    repr(option))
+
+
 class EndpointCaller(object):
     """L{EndpointCaller} objects are invoked via their instance methods to obtain results
     from the Rosette server described by the L{API} object from which they
@@ -421,7 +461,7 @@ class EndpointCaller(object):
         response = self.api.get_http(url, headers=headers)
         return self.__finish_result(response, "ping")
 
-    def call(self, parameters):
+    def call(self, parameters, paramtype=None):
         """Invokes the endpoint to which this L{EndpointCaller} is bound.
         Passes data and metadata specified by C{parameters} to the server
         endpoint to which this L{EndpointCaller} object is bound.  For all
@@ -438,24 +478,26 @@ class EndpointCaller(object):
         @param parameters: An object specifying the data,
         and possible metadata, to be processed by the endpoint.  See the
         details for those object types.
-        @type parameters: For C{name-translation}, L{NameTranslationParameters},
-        otherwise L{DocumentParameters} or L{str}
+        @type parameters: Parameters types or L{str} for document request.
+        @param paramtype: Required parameters type.
         @return: A python dictionary expressing the result of the invocation.
         """
+        if paramtype and not isinstance(parameters, paramtype):
+            raise RosetteException(
+                "incompatible",
+                "The parameters must be " + str(paramtype),
+                self.suburl)
 
-        if not isinstance(parameters, _DocumentParamSetBase):
-            if self.suburl != self.api.endpoints['NAME_SIMILARITY'] \
-                    and self.suburl != self.api.self.api.endpoints['NAME_TRANSLATION'] \
-                    and self.suburl != self.api.self.api.endpoints['NAME_DEDUPLICATION'] \
-                    and self.suburl != self.api.self.api.endpoints['ADDRESS_SIMILARITY']:
-                text = parameters
-                parameters = DocumentParameters()
-                parameters['content'] = text
-            else:
-                raise RosetteException(
-                    "incompatible",
-                    "Text-only input only works for DocumentParameter endpoints",
-                    self.suburl)
+        if type(parameters) == str:
+            text = parameters
+            parameters = DocumentParameters()
+            parameters['content'] = text
+
+        if not paramtype and not isinstance(parameters, DocumentParameters):
+            raise RosetteException(
+                "incompatible",
+                "The parameters must be string or DocumentParameters",
+                self.suburl)
 
         self.use_multipart = parameters.use_multipart
         url = self.service_url + self.suburl
@@ -495,7 +537,7 @@ class EndpointCaller(object):
                 _my_loads(rdata, response_headers), status)
         else:
             if self.debug:
-                headers[_CUSTOM_HEADER_PREFIX + 'Devel'] = True
+                headers[_CUSTOM_HEADER_PREFIX + 'Devel'] = 'true'
             self.logger.info('operate: ' + url)
             headers['Accept'] = _APPLICATION_JSON
             headers['Accept-Encoding'] = "gzip"
@@ -572,7 +614,9 @@ class API(object):
             'TEXT_EMBEDDING': 'semantics/vector',
             'TOKENS': 'tokens',
             'TOPICS': 'topics',
-            'TRANSLITERATION': 'transliteration'
+            'TRANSLITERATION': 'transliteration',
+            'EVENTS': 'events',
+            'RECORD_SIMILARITY': 'record-similarity'
         }
 
     def __del__(self):
@@ -897,7 +941,7 @@ class API(object):
         and possible metadata, to be processed by the name matcher.
         @type parameters: L{AddressSimilarityParameters}
         @return: A python dictionary containing the results of name matching."""
-        return EndpointCaller(self, self.endpoints['ADDRESS_SIMILARITY']).call(parameters)
+        return EndpointCaller(self, self.endpoints['ADDRESS_SIMILARITY']).call(parameters, AddressSimilarityParameters)
 
     def name_translation(self, parameters):
         """
@@ -907,7 +951,7 @@ class API(object):
         and possible metadata, to be processed by the name translator.
         @type parameters: L{NameTranslationParameters}
         @return: A python dictionary containing the results of name translation."""
-        return EndpointCaller(self, self.endpoints['NAME_TRANSLATION']).call(parameters)
+        return EndpointCaller(self, self.endpoints['NAME_TRANSLATION']).call(parameters, NameTranslationParameters)
 
     def translated_name(self, parameters):
         """ deprecated
@@ -926,7 +970,7 @@ class API(object):
         and possible metadata, to be processed by the name matcher.
         @type parameters: L{NameSimilarityParameters}
         @return: A python dictionary containing the results of name matching."""
-        return EndpointCaller(self, self.endpoints['NAME_SIMILARITY']).call(parameters)
+        return EndpointCaller(self, self.endpoints['NAME_SIMILARITY']).call(parameters, NameSimilarityParameters)
 
     def matched_name(self, parameters):
         """ deprecated
@@ -944,7 +988,16 @@ class API(object):
         as a threshold
         @type parameters: L{NameDeduplicationParameters}
         @return: A python dictionary containing the results of de-duplication"""
-        return EndpointCaller(self, self.endpoints['NAME_DEDUPLICATION']).call(parameters)
+        return EndpointCaller(self, self.endpoints['NAME_DEDUPLICATION']).call(parameters, NameDeduplicationParameters)
+
+    def record_similarity(self, parameters):
+        """
+        Create an L{EndpointCaller} to get similarity core between a list of records and call it.
+        @param parameters: An object specifying the data,
+        and possible metadata, to be processed by the record matcher.
+        @type parameters: L{RecordSimilarityParameters}
+        @return: A python dictionary containing the results of record matching."""
+        return EndpointCaller(self, self.endpoints['RECORD_SIMILARITY']).call(parameters, RecordSimilarityParameters)
 
     def text_embedding(self, parameters):
         """ deprecated
@@ -993,3 +1046,13 @@ class API(object):
         :return: A python dictionary containing the similar terms and their similarity
         """
         return EndpointCaller(self, self.endpoints['SIMILAR_TERMS']).call(parameters)
+
+    def events(self, parameters):
+        """
+        Create an L{EndpointCaller} to identify events found in the texts.
+        @param parameters: An object specifying the data,
+        and possible metadata, to be processed by the 'events' identifier.
+        @type parameters: L{DocumentParameters} or L{str}
+        @return: A python dictionary containing the results of event extraction.
+        """
+        return EndpointCaller(self, self.endpoints['EVENTS']).call(parameters)
